/*jshint node:true, laxcomma:true*/
'use strict';

var path       = require('path')
  , basename   = path.basename(__filename, '.js')
  , objectPath = require('object-path')
  , CSV        = require('csv-string')
  , filtrex    = require('filtrex')
  , evaluator = require('notevil')
  , transtype = require('transtype')
  , _ = require('lodash')
  , _s = require('underscore.string')
  , jsel = require('JSONSelect')
  ;

_.mixin(_s.exports());


var trimString = function trimString(str) {
  if (typeof str !== 'string') {
    return str;
  }
  return str.trim();
};

module.exports = function(options) {

  options = options || {};
  options.fieldname = options.fieldname ? options.fieldname : 'documentFields';
  options.schema = options.schema ? options.schema : {};

  if (typeof options.schema !== 'object') {
    options.schema = {};
  }

  return function (input, submit) {
    var values = {}, text = '';
    Object.keys(options.schema).forEach(function (key) {

      //
      // .default
      //
      if (options.schema[key].default) {
        values[key] = options.schema[key].default;
      }
      else {
        values[key] = '';
      }

      //
      // .template
      //
      if (options.schema[key].template) {
        try {
          values[key] = _.template(options.schema[key].template, values);
        }
        catch(e) {
          values[key] = e.toString();
        }
      }

      //
      // .select
      //
      if (options.schema[key].select) {
        var vals;
        if (typeof options.schema[key].select === 'string') {
          try {
            vals = jsel(options.schema[key].path, input);
          }
          catch(e) {
            vals = e.toString();
          }
        }
        else if (Array.isArray(options.schema[key].select)) {
          vals = options.schema[key].select.map(function(i) {
            try {
              return jsel(options.schema[key].path, input);
            }
            catch(e) {
              return e.toString();
            }
          }).filter(function(i) { return (i !== '' && i !== undefined && i !== null); });
          if (options.schema[key].coalesce) {
            vals = vals[0];
          }
        }
        if (vals !== undefined && vals !== null) {
          values[key] = vals;
        }
      }


      //
      // .path
      //
      if (options.schema[key].path) {
        var valp;
        if (typeof options.schema[key].path === 'string') {
          valp = objectPath.get(input, options.schema[key].path);
        }
        else if (Array.isArray(options.schema[key].path)) {
          if (options.schema[key].coalesce) {
            valp = objectPath.coalesce(input, options.schema[key].path);
          }
          else {
            valp = options.schema[key].path.map(function(i) { return objectPath.get(input, i); }).filter(function(i) { return (i !== '' && i !== undefined && i !== null); });
          }
        }
        if (valp !== undefined && valp !== null) {
          values[key] = valp;
        }
      }

      //
      // .glue
      //
      if (options.schema[key].glue &&
          Array.isArray(values[key])) {
        values[key] = values[key].join(options.schema[key].glue);
      }

      //
      // .transform
      //
      if (options.schema[key].transform && typeof options.schema[key].transform === 'string') {
        var exp = 'return input.' + options.schema[key].transform + '.value()';
        var func1 = evaluator.Function('input', exp);
        values[key] = func1(_.chain(values[key]));
      }

      //
      // .textizer
      //
      if (options.schema[key].textizer && typeof options.schema[key].textizer === 'string') {
        var exp2 = 'return input.' + options.schema[key].textizer + '.value()';
        var func2 = evaluator.Function('input', exp2);
        text += func2(_.chain(values[key]));
      }

      //
      // .separator
      //
      if (options.schema[key].separator) {
        values[key] = CSV.parse(values[key], options.schema[key].separator).shift().map(trimString).filter(function(i) { return (i !== '' && i !== undefined && i !== null); });
      }

      //
      // .type && .pattern
      //
      if (options.schema[key].type &&
          typeof options.schema[key].type === 'string') {
        values[key] = transtype(options.schema[key].type, values[key], options.schema[key].pattern);
      }

      //
      // .compute
      //
      if (options.schema[key].compute &&
          typeof options.schema[key].compute === 'string') {
        var func = filtrex(options.schema[key].compute);
        values[key] = func(values);
      }


      //
      // .mapping
      //
      if (options.schema[key].mapping) {
        if ((typeof values[key] === 'string' || typeof values[key] === 'number') && options.schema[key].mapping[values[key]]) {
          values[key] = options.schema[key].mapping[values[key]];
        }
        else if (Array.isArray(values[key])) {
          values[key] = values[key].map(function(i) {
             if ((typeof i === 'string' || typeof values[key] === 'number') && options.schema[key].mapping[i]) {
               values[key] = options.schema[key].mapping[i];
             }
          });
        }
      }

      //
      // .noindex
      //
      if (options.schema[key].noindex !== true) {
        if (typeof values[key] === 'string') {
          // Truncate resulting strings, to allow the indexing by Mongo
          // (Mongo 2.6 does not insert the document if any indexing does not work)
          values[key] = values[key].toString().truncate(250).toString();
        }
      }

      //
      // .required
      //
      if (options.schema[key].required && (values[key] === '' || values[key] === undefined || values[key] === null || values[key].length === 0)) {
        return submit(new Error(key + ' is required'));
      }
    });
    input[options.fieldname] = values;
    if (text !== '') {
      input.text = text;
    }
    submit(null, input);
  };
};
